@@ -20,9 +20,7 @@ namespace standardBML
  * model for C3 photosynthesis, along with simple 1D gas diffusion, define a set
  * of coupled equations with three unknowns: net assimilation, intercellular CO2
  * concetration, and stomatal conductance. This function attempts to numerically
- * determine these unknowns using a fixed-point iteration method. Beware: the
- * code often fails to find the true solution, and some parts of the models are
- * implemented incorrectly.
+ * determine these unknowns using the secant method.
  *
  * See the original WIMOVAC paper for a description of the assimilation model
  * and additional citations: [Humphries, S. W. & Long, S. P. Bioinformatics
@@ -31,37 +29,39 @@ namespace standardBML
  * ### BioCro module implementation
  *
  * In BioCro, we use the following names for this model's input quantities:
- * - ``'Qabs'`` for the absorbed quantum flux density of photosynthetically active radiation
- * - ``'Tleaf'`` for the leaf temperature
- * - ``'temp'`` for the ambient temperature
- * - ``'rh'`` for the atmospheric relative humidity
- * - ``'vmax1'`` for the rubisco carboxylation rate at 25 degrees C
- * - ``'jmax'`` for the electron transport rate
- * - ``'tpu_rate_max'`` for the maximum triose phosphate utilization rate
- * - ``'Rd'`` for the respiration rate at 25 degrees C
+ * - ``'atmospheric_pressure'`` for the local atmospheric_pressure
  * - ``'b0'`` for the Ball-Berry intercept
  * - ``'b1'`` for the Ball-Berry slope
- * - ``'Gs_min'`` for the minimum stomatal conductance (only used when applying water stress via stomatal conductance)
  * - ``'Catm'`` for the atmospheric CO2 concentration
- * - ``'atmospheric_pressure'`` for the local atmospheric_pressure
- * - ``'O2'`` for the atmospheric O2 concentration
- * - ``'theta'`` for the ???
- * - ``'StomataWS'`` for the water stress factor
  * - ``'electrons_per_carboxylation'`` for the number of electrons per carboxylation event
  * - ``'electrons_per_oxygenation'`` for the number of electrons per oxygenation event
  * - ``'gbw'`` for the boundary layer conductance to water vapor. For an
  *   isolated leaf, this should be the leaf boundary layer conductance; for a
  *   leaf within a canopy, this should be the total conductance including the
  *   leaf and canopy boundary layer conductances.
+ * - ``'Gs_min'`` for the minimum stomatal conductance (only used when applying water stress via stomatal conductance)
+ * - ``'Jmax_at_25'`` for the maximum electron transport rate at 25 degrees C
+ * - ``'O2'`` for the atmospheric O2 concentration
+ * - ``'Qabs'`` for the absorbed quantum flux density of photosynthetically active radiation
+ * - ``'rh'`` for the atmospheric relative humidity
+ * - ``'RL_at_25'`` for the rate of non-photorespiratory CO2 release in the light at 25 degrees C
+ * - ``'StomataWS'`` for the water stress factor
+ * - ``'temp'`` for the ambient temperature
+ * - ``'theta'`` for the ???
+ * - ``'Tleaf'`` for the leaf temperature
+ * - ``'Tp_at_25'`` for the maximum triose phosphate utilization rate at 25 degrees C
+ * - ``'Vcmax_at_25'`` for the rubisco carboxylation rate at 25 degrees C
  *
  * We use the following names for the model's output quantities:
  * - ``'Assim'`` for the net CO2 assimilation rate
- * - ``'Gs'`` for the stomatal conductance for H2O
- * - ``'Cs'`` for the CO2 concentration at the leaf surface
- * - ``'RHs'`` for the relative humidity at the leaf surface
- * - ``'Ci'`` for the intercellular CO2 concentration
- * - ``'GrossAssim'`` for the gross CO2 assimilation rate
+ * - ``'Assim_check'`` for an indicator of whether the loop converged
  * - ``'Assim_conductance'`` for the maximum net assimilation rate limited by conductance
+ * - ``'Ci'`` for the intercellular CO2 concentration
+ * - ``'Cs'`` for the CO2 concentration at the leaf surface
+ * - ``'GrossAssim'`` for the gross CO2 assimilation rate
+ * - ``'Gs'`` for the stomatal conductance for H2O
+ * - ``'RHs'`` for the relative humidity at the leaf surface
+ * - ``'RL'`` for the rate of non-photorespiratory CO2 release in the light
  * - ``'Rp'`` for the rate of photorespiration
  * - ``'iterations'`` for the number of iterations required for the convergence loop
  */
@@ -85,7 +85,7 @@ class c3_assimilation : public direct_module
           Gs_min{get_input(input_quantities, "Gs_min")},
           Gstar_c{get_input(input_quantities, "Gstar_c")},
           Gstar_Ea{get_input(input_quantities, "Gstar_Ea")},
-          jmax{get_input(input_quantities, "jmax")},
+          Jmax_at_25{get_input(input_quantities, "Jmax_at_25")},
           Jmax_c{get_input(input_quantities, "Jmax_c")},
           Jmax_Ea{get_input(input_quantities, "Jmax_Ea")},
           Kc_c{get_input(input_quantities, "Kc_c")},
@@ -97,33 +97,35 @@ class c3_assimilation : public direct_module
           phi_PSII_1{get_input(input_quantities, "phi_PSII_1")},
           phi_PSII_2{get_input(input_quantities, "phi_PSII_2")},
           Qabs{get_input(input_quantities, "Qabs")},
-          Rd{get_input(input_quantities, "Rd")},
-          Rd_c{get_input(input_quantities, "Rd_c")},
-          Rd_Ea{get_input(input_quantities, "Rd_Ea")},
           rh{get_input(input_quantities, "rh")},
+          RL_at_25{get_input(input_quantities, "RL_at_25")},
+          RL_c{get_input(input_quantities, "RL_c")},
+          RL_Ea{get_input(input_quantities, "RL_Ea")},
           StomataWS{get_input(input_quantities, "StomataWS")},
           Tambient{get_input(input_quantities, "temp")},
           theta_0{get_input(input_quantities, "theta_0")},
           theta_1{get_input(input_quantities, "theta_1")},
           theta_2{get_input(input_quantities, "theta_2")},
           Tleaf{get_input(input_quantities, "Tleaf")},
+          Tp_at_25{get_input(input_quantities, "Tp_at_25")},
           Tp_c{get_input(input_quantities, "Tp_c")},
           Tp_Ha{get_input(input_quantities, "Tp_Ha")},
           Tp_Hd{get_input(input_quantities, "Tp_Hd")},
           Tp_S{get_input(input_quantities, "Tp_S")},
-          tpu_rate_max{get_input(input_quantities, "tpu_rate_max")},
+          Vcmax_at_25{get_input(input_quantities, "Vcmax_at_25")},
           Vcmax_c{get_input(input_quantities, "Vcmax_c")},
           Vcmax_Ea{get_input(input_quantities, "Vcmax_Ea")},
-          vmax1{get_input(input_quantities, "vmax1")},
 
           // Get pointers to output quantities
           Assim_op{get_op(output_quantities, "Assim")},
-          Gs_op{get_op(output_quantities, "Gs")},
-          Cs_op{get_op(output_quantities, "Cs")},
-          RHs_op{get_op(output_quantities, "RHs")},
-          Ci_op{get_op(output_quantities, "Ci")},
-          GrossAssim_op{get_op(output_quantities, "GrossAssim")},
+          Assim_check_op{get_op(output_quantities, "Assim_check")},
           Assim_conductance_op{get_op(output_quantities, "Assim_conductance")},
+          Ci_op{get_op(output_quantities, "Ci")},
+          Cs_op{get_op(output_quantities, "Cs")},
+          GrossAssim_op{get_op(output_quantities, "GrossAssim")},
+          Gs_op{get_op(output_quantities, "Gs")},
+          RHs_op{get_op(output_quantities, "RHs")},
+          RL_op{get_op(output_quantities, "RL")},
           Rp_op{get_op(output_quantities, "Rp")},
           iterations_op{get_op(output_quantities, "iterations")}
     {
@@ -145,7 +147,7 @@ class c3_assimilation : public direct_module
     double const& Gs_min;
     double const& Gstar_c;
     double const& Gstar_Ea;
-    double const& jmax;
+    double const& Jmax_at_25;
     double const& Jmax_c;
     double const& Jmax_Ea;
     double const& Kc_c;
@@ -157,33 +159,35 @@ class c3_assimilation : public direct_module
     double const& phi_PSII_1;
     double const& phi_PSII_2;
     double const& Qabs;
-    double const& Rd;
-    double const& Rd_c;
-    double const& Rd_Ea;
     double const& rh;
+    double const& RL_at_25;
+    double const& RL_c;
+    double const& RL_Ea;
     double const& StomataWS;
     double const& Tambient;
     double const& theta_0;
     double const& theta_1;
     double const& theta_2;
     double const& Tleaf;
+    double const& Tp_at_25;
     double const& Tp_c;
     double const& Tp_Ha;
     double const& Tp_Hd;
     double const& Tp_S;
-    double const& tpu_rate_max;
+    double const& Vcmax_at_25;
     double const& Vcmax_c;
     double const& Vcmax_Ea;
-    double const& vmax1;
 
     // Pointers to output quantities
     double* Assim_op;
-    double* Gs_op;
-    double* Cs_op;
-    double* RHs_op;
-    double* Ci_op;
-    double* GrossAssim_op;
+    double* Assim_check_op;
     double* Assim_conductance_op;
+    double* Ci_op;
+    double* Cs_op;
+    double* GrossAssim_op;
+    double* Gs_op;
+    double* RHs_op;
+    double* RL_op;
     double* Rp_op;
     double* iterations_op;
 
@@ -205,7 +209,7 @@ string_vector c3_assimilation::get_inputs()
         "Gs_min",                       // mol / m^2 / s
         "Gstar_c",                      // dimensionless
         "Gstar_Ea",                     // J / mol
-        "jmax",                         // micromol / m^2 / s
+        "Jmax_at_25",                   // micromol / m^2 / s
         "Jmax_c",                       // dimensionless
         "Jmax_Ea",                      // J / mol
         "Kc_c",                         // dimensionless
@@ -217,24 +221,24 @@ string_vector c3_assimilation::get_inputs()
         "phi_PSII_1",                   // (degrees C)^(-1)
         "phi_PSII_2",                   // (degrees C)^(-2)
         "Qabs",                         // micromol / m^2 / s
-        "Rd",                           // micromol / m^2 / s
-        "Rd_c",                         // dimensionless
-        "Rd_Ea",                        // J / mol
         "rh",                           // dimensionless
+        "RL_at_25",                     // micromol / m^2 / s
+        "RL_c",                         // dimensionless
+        "RL_Ea",                        // J / mol
         "StomataWS",                    // dimensionless
         "temp",                         // degrees C
         "theta_0",                      // dimensionless
         "theta_1",                      // (degrees C)^(-1)
         "theta_2",                      // (degrees C)^(-2)
         "Tleaf",                        // degrees C
+        "Tp_at_25",                     // micromol / m^2 / s
         "Tp_c",                         // dimensionless
         "Tp_Ha",                        // J / mol
         "Tp_Hd",                        // J / mol
         "Tp_S",                         // J / K / mol
-        "tpu_rate_max",                 // micromol / m^2 / s
+        "Vcmax_at_25",                  // micromol / m^2 / s
         "Vcmax_c",                      // dimensionless
         "Vcmax_Ea",                     // J / mol
-        "vmax1"                         // micromol / m^2 / s
     };
 }
 
@@ -242,12 +246,14 @@ string_vector c3_assimilation::get_outputs()
 {
     return {
         "Assim",              // micromol / m^2 / s
-        "Gs",                 // mol / m^2 / s
-        "Cs",                 // micromol / m^2 / s
-        "RHs",                // dimensionless from Pa / Pa
-        "Ci",                 // micromol / mol
-        "GrossAssim",         // micromol / m^2 / s
+        "Assim_check",        // micromol / m^2 / s
         "Assim_conductance",  // micromol / m^2 / s
+        "Ci",                 // micromol / mol
+        "Cs",                 // micromol / m^2 / s
+        "GrossAssim",         // micromol / m^2 / s
+        "Gs",                 // mol / m^2 / s
+        "RHs",                // dimensionless from Pa / Pa
+        "RL",                 // micromol / m^2 / s
         "Rp",                 // micromol / m^2 / s
         "iterations"          // not a physical quantity
     };
@@ -268,8 +274,8 @@ void c3_assimilation::do_operation() const
         phi_PSII_0,
         phi_PSII_1,
         phi_PSII_2,
-        Rd_c,
-        Rd_Ea,
+        RL_c,
+        RL_Ea,
         theta_0,
         theta_1,
         theta_2,
@@ -286,10 +292,10 @@ void c3_assimilation::do_operation() const
         Tleaf,
         Tambient,
         rh,
-        vmax1,
-        jmax,
-        tpu_rate_max,
-        Rd,
+        Vcmax_at_25,
+        Jmax_at_25,
+        Tp_at_25,
+        RL_at_25,
         b0,
         b1,
         Gs_min,
@@ -303,13 +309,15 @@ void c3_assimilation::do_operation() const
         gbw);
 
     // Update the output quantity list
-    update(Assim_op, c3_results.Assim);
-    update(Gs_op, c3_results.Gs);
-    update(Cs_op, c3_results.Cs);
-    update(RHs_op, c3_results.RHs);
-    update(Ci_op, c3_results.Ci);
-    update(GrossAssim_op, c3_results.GrossAssim);
+    update(Assim_check_op, c3_results.Assim_check);
     update(Assim_conductance_op, c3_results.Assim_conductance);
+    update(Assim_op, c3_results.Assim);
+    update(Ci_op, c3_results.Ci);
+    update(Cs_op, c3_results.Cs);
+    update(GrossAssim_op, c3_results.GrossAssim);
+    update(Gs_op, c3_results.Gs);
+    update(RHs_op, c3_results.RHs);
+    update(RL_op, c3_results.RL);
     update(Rp_op, c3_results.Rp);
     update(iterations_op, c3_results.iterations);
 }
