@@ -37,7 +37,7 @@ class c3_leaf_photosynthesis : public direct_module
           Gstar_c{get_input(input_quantities, "Gstar_c")},
           Gstar_Ea{get_input(input_quantities, "Gstar_Ea")},
           height{get_input(input_quantities, "height")},
-          jmax{get_input(input_quantities, "jmax")},
+          Jmax_at_25{get_input(input_quantities, "Jmax_at_25")},
           Jmax_c{get_input(input_quantities, "Jmax_c")},
           Jmax_Ea{get_input(input_quantities, "Jmax_Ea")},
           Kc_c{get_input(input_quantities, "Kc_c")},
@@ -49,22 +49,22 @@ class c3_leaf_photosynthesis : public direct_module
           phi_PSII_0{get_input(input_quantities, "phi_PSII_0")},
           phi_PSII_1{get_input(input_quantities, "phi_PSII_1")},
           phi_PSII_2{get_input(input_quantities, "phi_PSII_2")},
-          Rd{get_input(input_quantities, "Rd")},
-          Rd_c{get_input(input_quantities, "Rd_c")},
-          Rd_Ea{get_input(input_quantities, "Rd_Ea")},
           rh{get_input(input_quantities, "rh")},
+          RL_at_25{get_input(input_quantities, "RL_at_25")},
+          RL_c{get_input(input_quantities, "RL_c")},
+          RL_Ea{get_input(input_quantities, "RL_Ea")},
           StomataWS{get_input(input_quantities, "StomataWS")},
           theta_0{get_input(input_quantities, "theta_0")},
           theta_1{get_input(input_quantities, "theta_1")},
           theta_2{get_input(input_quantities, "theta_2")},
+          Tp_at_25{get_input(input_quantities, "Tp_at_25")},
           Tp_c{get_input(input_quantities, "Tp_c")},
           Tp_Ha{get_input(input_quantities, "Tp_Ha")},
           Tp_Hd{get_input(input_quantities, "Tp_Hd")},
           Tp_S{get_input(input_quantities, "Tp_S")},
-          tpu_rate_max{get_input(input_quantities, "tpu_rate_max")},
+          Vcmax_at_25{get_input(input_quantities, "Vcmax_at_25")},
           Vcmax_c{get_input(input_quantities, "Vcmax_c")},
           Vcmax_Ea{get_input(input_quantities, "Vcmax_Ea")},
-          vmax1{get_input(input_quantities, "vmax1")},
           windspeed{get_input(input_quantities, "windspeed")},
 
           // Get pointers to output quantities
@@ -79,6 +79,7 @@ class c3_leaf_photosynthesis : public direct_module
           leaf_temperature_op{get_op(output_quantities, "leaf_temperature")},
           RHs_op{get_op(output_quantities, "RHs")},
           RH_canopy_op{get_op(output_quantities, "RH_canopy")},
+          RL_op{get_op(output_quantities, "RL")},
           Rp_op{get_op(output_quantities, "Rp")},
           TransR_op{get_op(output_quantities, "TransR")}
     {
@@ -105,7 +106,7 @@ class c3_leaf_photosynthesis : public direct_module
     double const& Gstar_c;
     double const& Gstar_Ea;
     double const& height;
-    double const& jmax;
+    double const& Jmax_at_25;
     double const& Jmax_c;
     double const& Jmax_Ea;
     double const& Kc_c;
@@ -117,22 +118,22 @@ class c3_leaf_photosynthesis : public direct_module
     double const& phi_PSII_0;
     double const& phi_PSII_1;
     double const& phi_PSII_2;
-    double const& Rd;
-    double const& Rd_c;
-    double const& Rd_Ea;
     double const& rh;
+    double const& RL_at_25;
+    double const& RL_c;
+    double const& RL_Ea;
     double const& StomataWS;
     double const& theta_0;
     double const& theta_1;
     double const& theta_2;
+    double const& Tp_at_25;
     double const& Tp_c;
     double const& Tp_Ha;
     double const& Tp_Hd;
     double const& Tp_S;
-    double const& tpu_rate_max;
+    double const& Vcmax_at_25;
     double const& Vcmax_c;
     double const& Vcmax_Ea;
-    double const& vmax1;
     double const& windspeed;
 
     // Pointers to output quantities
@@ -147,6 +148,7 @@ class c3_leaf_photosynthesis : public direct_module
     double* leaf_temperature_op;
     double* RHs_op;
     double* RH_canopy_op;
+    double* RL_op;
     double* Rp_op;
     double* TransR_op;
 
