@@ -10,10 +10,10 @@ photosynthesis_outputs c3photoC(
     double const Tleaf,
     double const Tambient,
     double const RH,
-    double const Vcmax0,
-    double const Jmax0,
+    double const Vcmax_at_25,
+    double const Jmax_at_25,
     double const TPU_rate_max,
-    double const Rd0,
+    double const RL_at_25,
     double const bb0,
     double const bb1,
     double const Gs_min,
