@@ -19,9 +19,7 @@ namespace standardBML
  * photosynthesis, along with simple 1D gas diffusion, define a set of coupled
  * equations with three unknowns: net assimilation, intercellular CO2
  * concetration, and stomatal conductance. This function attempts to numerically
- * determine these unknowns using a fixed-point iteration method. Beware: the
- * code often fails to find the true solution, and some parts of the models are
- * implemented incorrectly.
+ * determine these unknowns using the secant method.
  *
  * See the original WIMOVAC paper for a description of the assimilation model
  * and additional citations: [Humphries, S. W. & Long, S. P. Bioinformatics
@@ -30,37 +28,39 @@ namespace standardBML
  * ### BioCro module implementation
  *
  * In BioCro, we use the following names for this model's input quantities:
- * - ``'Qp'`` for the incident quantum flux density of photosynthetically active radiation
- * - ``'Tleaf'`` for the leaf temperature
- * - ``'temp'`` for the ambient temperature
- * - ``'rh'`` for the atmospheric relative humidity
- * - ``'vmax'`` for the rubisco carboxylation rate at 25 degrees C
  * - ``'alpha'`` for the initial slope of the photosynthetic light response
- * - ``'kparm'`` for the initial slope of the photosynthetic CO2 response
- * - ``'theta'`` for the first quadratic mixing parameter
- * - ``'beta'`` for the second quandratic mixing parameter
- * - ``'Rd'`` for the respiration rate at 25 degrees C
+ * - ``'atmospheric_pressure'`` for the local atmospheric pressure
  * - ``'b0'`` for the Ball-Berry intercept
  * - ``'b1'`` for the Ball-Berry slope
- * - ``'Gs_min'`` for the minimum stomatal conductance (only used when applying water stress via stomatal conductance)
- * - ``'StomataWS'`` for the water stress factor
+ * - ``'beta'`` for the second quandratic mixing parameter
  * - ``'Catm'`` for the atmospheric CO2 concentration
- * - ``'atmospheric_pressure'`` for the local atmospheric pressure
- * - ``'upperT'`` for the high temperature cutoff for rubisco activity
- * - ``'lowerT'`` for the low temperature cutoff for rubisco activity
  * - ``'gbw'`` for the boundary layer conductance to water vapor. For an
  *   isolated leaf, this should be the leaf boundary layer conductance; for a
  *   leaf within a canopy, this should be the total conductance including the
  *   leaf and canopy boundary layer conductances.
+ * - ``'Gs_min'`` for the minimum stomatal conductance (only used when applying water stress via stomatal conductance)
+ * - ``'kparm'`` for the initial slope of the photosynthetic CO2 response
+ * - ``'lowerT'`` for the low temperature cutoff for rubisco activity
+ * - ``'Qp'`` for the incident quantum flux density of photosynthetically active radiation
+ * - ``'rh'`` for the atmospheric relative humidity
+ * - ``'RL_at_25'`` for the rate of non-photorespiratory CO2 release in the light at 25 degrees C
+ * - ``'StomataWS'`` for the water stress factor
+ * - ``'temp'`` for the ambient temperature
+ * - ``'theta'`` for the first quadratic mixing parameter
+ * - ``'Tleaf'`` for the leaf temperature
+ * - ``'upperT'`` for the high temperature cutoff for rubisco activity
+ * - ``'Vcmax_at_25'`` for the rubisco carboxylation rate at 25 degrees C
  *
  * We use the following names for the model's output quantities:
  * - ``'Assim'`` for the net CO2 assimilation rate
- * - ``'Gs'`` for the stomatal conductance for H2O
- * - ``'Cs'`` for the CO2 concentration at the leaf surface
- * - ``'RHs'`` for the relative humidity at the leaf surface
- * - ``'Ci'`` for the intercellular CO2 concentration
- * - ``'GrossAssim'`` for the gross CO2 assimilation rate
+ * - ``'Assim_check'`` for an indicator of whether the loop converged
  * - ``'Assim_conductance'`` for the maximum net assimilation rate limited by conductance
+ * - ``'Ci'`` for the intercellular CO2 concentration
+ * - ``'Cs'`` for the CO2 concentration at the leaf surface
+ * - ``'GrossAssim'`` for the gross CO2 assimilation rate
+ * - ``'Gs'`` for the stomatal conductance for H2O
+ * - ``'RHs'`` for the relative humidity at the leaf surface
+ * - ``'RL'`` for the rate of non-photorespiratory CO2 release in the light
  * - ``'Rp'`` for the rate of photorespiration
  * - ``'iterations'`` for the number of iterations required for the convergence loop
  */
@@ -73,34 +73,36 @@ class c4_assimilation : public direct_module
         : direct_module{},
 
           // Get pointers to input quantities
-          Qp{get_input(input_quantities, "Qp")},
-          Tleaf{get_input(input_quantities, "Tleaf")},
-          Tambient{get_input(input_quantities, "temp")},
-          rh{get_input(input_quantities, "rh")},
-          vmax{get_input(input_quantities, "vmax")},
           alpha{get_input(input_quantities, "alpha")},
-          kparm{get_input(input_quantities, "kparm")},
-          theta{get_input(input_quantities, "theta")},
-          beta{get_input(input_quantities, "beta")},
-          Rd{get_input(input_quantities, "Rd")},
+          atmospheric_pressure{get_input(input_quantities, "atmospheric_pressure")},
           b0{get_input(input_quantities, "b0")},
           b1{get_input(input_quantities, "b1")},
-          Gs_min{get_input(input_quantities, "Gs_min")},
-          StomataWS{get_input(input_quantities, "StomataWS")},
+          beta{get_input(input_quantities, "beta")},
           Catm{get_input(input_quantities, "Catm")},
-          atmospheric_pressure{get_input(input_quantities, "atmospheric_pressure")},
-          upperT{get_input(input_quantities, "upperT")},
-          lowerT{get_input(input_quantities, "lowerT")},
           gbw{get_input(input_quantities, "gbw")},
+          Gs_min{get_input(input_quantities, "Gs_min")},
+          kparm{get_input(input_quantities, "kparm")},
+          lowerT{get_input(input_quantities, "lowerT")},
+          Qp{get_input(input_quantities, "Qp")},
+          rh{get_input(input_quantities, "rh")},
+          RL_at_25{get_input(input_quantities, "RL_at_25")},
+          StomataWS{get_input(input_quantities, "StomataWS")},
+          Tambient{get_input(input_quantities, "temp")},
+          theta{get_input(input_quantities, "theta")},
+          Tleaf{get_input(input_quantities, "Tleaf")},
+          upperT{get_input(input_quantities, "upperT")},
+          Vcmax_at_25{get_input(input_quantities, "Vcmax_at_25")},
 
           // Get pointers to output quantities
           Assim_op{get_op(output_quantities, "Assim")},
-          Gs_op{get_op(output_quantities, "Gs")},
-          Cs_op{get_op(output_quantities, "Cs")},
-          RHs_op{get_op(output_quantities, "RHs")},
-          Ci_op{get_op(output_quantities, "Ci")},
-          GrossAssim_op{get_op(output_quantities, "GrossAssim")},
+          Assim_check_op{get_op(output_quantities, "Assim_check")},
           Assim_conductance_op{get_op(output_quantities, "Assim_conductance")},
+          Ci_op{get_op(output_quantities, "Ci")},
+          Cs_op{get_op(output_quantities, "Cs")},
+          GrossAssim_op{get_op(output_quantities, "GrossAssim")},
+          Gs_op{get_op(output_quantities, "Gs")},
+          RHs_op{get_op(output_quantities, "RHs")},
+          RL_op{get_op(output_quantities, "RL")},
           Rp_op{get_op(output_quantities, "Rp")},
           iterations_op{get_op(output_quantities, "iterations")}
     {
@@ -111,34 +113,36 @@ class c4_assimilation : public direct_module
 
    private:
     // References to input quantities
-    double const& Qp;
-    double const& Tleaf;
-    double const& Tambient;
-    double const& rh;
-    double const& vmax;
     double const& alpha;
-    double const& kparm;
-    double const& theta;
-    double const& beta;
-    double const& Rd;
+    double const& atmospheric_pressure;
     double const& b0;
     double const& b1;
-    double const& Gs_min;
-    double const& StomataWS;
+    double const& beta;
     double const& Catm;
-    double const& atmospheric_pressure;
-    double const& upperT;
-    double const& lowerT;
     double const& gbw;
+    double const& Gs_min;
+    double const& kparm;
+    double const& lowerT;
+    double const& Qp;
+    double const& rh;
+    double const& RL_at_25;
+    double const& StomataWS;
+    double const& Tambient;
+    double const& theta;
+    double const& Tleaf;
+    double const& upperT;
+    double const& Vcmax_at_25;
 
     // Pointers to output quantities
     double* Assim_op;
-    double* Gs_op;
-    double* Cs_op;
-    double* RHs_op;
-    double* Ci_op;
-    double* GrossAssim_op;
+    double* Assim_check_op;
     double* Assim_conductance_op;
+    double* Ci_op;
+    double* Cs_op;
+    double* GrossAssim_op;
+    double* Gs_op;
+    double* RHs_op;
+    double* RL_op;
     double* Rp_op;
     double* iterations_op;
 
@@ -149,25 +153,25 @@ class c4_assimilation : public direct_module
 string_vector c4_assimilation::get_inputs()
 {
     return {
-        "Qp",                    // micromol / m^2 / s
-        "Tleaf",                 // degrees C
-        "temp",                  // degrees C
-        "rh",                    // dimensionless
-        "vmax",                  // micromol / m^2 / s
         "alpha",                 // mol / mol
-        "kparm",                 // mol / mol
-        "theta",                 // dimensionless
-        "beta",                  // dimensionless
-        "Rd",                    // micromol / m^2 / s
+        "atmospheric_pressure",  // Pa
         "b0",                    // mol / m^2 / s
         "b1",                    // dimensionless
-        "Gs_min",                // mol / m^2 / s
-        "StomataWS",             // dimensionless
+        "beta",                  // dimensionless
         "Catm",                  // micromol / mol
-        "atmospheric_pressure",  // Pa
-        "upperT",                // degrees C
+        "gbw",                   // mol / m^2 / s
+        "Gs_min",                // mol / m^2 / s
+        "kparm",                 // mol / mol
         "lowerT",                // degrees C
-        "gbw"                    // mol / m^2 / s
+        "Qp",                    // micromol / m^2 / s
+        "rh",                    // dimensionless
+        "RL_at_25",              // micromol / m^2 / s
+        "StomataWS",             // dimensionless
+        "temp",                  // degrees C
+        "theta",                 // dimensionless
+        "Tleaf",                 // degrees C
+        "upperT",                // degrees C
+        "Vcmax_at_25"            // micromol / m^2 / s
     };
 }
 
@@ -175,12 +179,14 @@ string_vector c4_assimilation::get_outputs()
 {
     return {
         "Assim",              // micromol / m^2 / s
-        "Gs",                 // mol / m^2 / s
-        "Cs",                 // micromol / m^2 / s
-        "RHs",                // dimensionless from Pa / Pa
-        "Ci",                 // micromol / mol
-        "GrossAssim",         // micromol / m^2 / s
+        "Assim_check",        // micromol / m^2 / s
         "Assim_conductance",  // micromol / m^2 / s
+        "Ci",                 // micromol / mol
+        "Cs",                 // micromol / m^2 / s
+        "GrossAssim",         // micromol / m^2 / s
+        "Gs",                 // mol / m^2 / s
+        "RHs",                // dimensionless from Pa / Pa
+        "RL",                 // micromol / m^2 / s
         "Rp",                 // micromol / m^2 / s
         "iterations"          // not a physical quantity
     };
@@ -193,12 +199,12 @@ void c4_assimilation::do_operation() const
         Tleaf,
         Tambient,
         rh,
-        vmax,
+        Vcmax_at_25,
         alpha,
         kparm,
         theta,
         beta,
-        Rd,
+        RL_at_25,
         b0,
         b1,
         Gs_min,
@@ -211,12 +217,14 @@ void c4_assimilation::do_operation() const
 
     // Update the output quantity list
     update(Assim_op, c4_results.Assim);
-    update(Gs_op, c4_results.Gs);
-    update(Cs_op, c4_results.Cs);
-    update(RHs_op, c4_results.RHs);
-    update(Ci_op, c4_results.Ci);
-    update(GrossAssim_op, c4_results.GrossAssim);
+    update(Assim_check_op, c4_results.Assim_check);
     update(Assim_conductance_op, c4_results.Assim_conductance);
+    update(Ci_op, c4_results.Ci);
+    update(Cs_op, c4_results.Cs);
+    update(GrossAssim_op, c4_results.GrossAssim);
+    update(Gs_op, c4_results.Gs);
+    update(RHs_op, c4_results.RHs);
+    update(RL_op, c4_results.RL);
     update(Rp_op, c4_results.Rp);
     update(iterations_op, c4_results.iterations);
 }
