@@ -28,17 +28,18 @@ class c4_canopy : public direct_module
           chil{get_input(input_quantities, "chil")},
           cosine_zenith_angle{get_input(input_quantities, "cosine_zenith_angle")},
           gbw_canopy{get_input(input_quantities, "gbw_canopy")},
+          growth_respiration_fraction{get_input(input_quantities, "growth_respiration_fraction")},
           Gs_min{get_input(input_quantities, "Gs_min")},
+          k_diffuse{get_input(input_quantities, "k_diffuse")},
           kparm{get_input(input_quantities, "kparm")},
           kpLN{get_input(input_quantities, "kpLN")},
-          k_diffuse{get_input(input_quantities, "k_diffuse")},
           lai{get_input(input_quantities, "lai")},
-          LeafN{get_input(input_quantities, "LeafN")},
-          leafwidth{get_input(input_quantities, "leafwidth")},
           leaf_reflectance_nir{get_input(input_quantities, "leaf_reflectance_nir")},
           leaf_reflectance_par{get_input(input_quantities, "leaf_reflectance_par")},
           leaf_transmittance_nir{get_input(input_quantities, "leaf_transmittance_nir")},
           leaf_transmittance_par{get_input(input_quantities, "leaf_transmittance_par")},
+          LeafN{get_input(input_quantities, "LeafN")},
+          leafwidth{get_input(input_quantities, "leafwidth")},
           lnfun{get_input(input_quantities, "lnfun")},
           lowerT{get_input(input_quantities, "lowerT")},
           nalphab0{get_input(input_quantities, "nalphab0")},
@@ -55,22 +56,24 @@ class c4_canopy : public direct_module
           nvmaxb1{get_input(input_quantities, "nvmaxb1")},
           par_energy_content{get_input(input_quantities, "par_energy_content")},
           par_energy_fraction{get_input(input_quantities, "par_energy_fraction")},
-          Rd{get_input(input_quantities, "Rd")},
           rh{get_input(input_quantities, "rh")},
+          RL_at_25{get_input(input_quantities, "RL_at_25")},
           solar{get_input(input_quantities, "solar")},
           StomataWS{get_input(input_quantities, "StomataWS")},
           temp{get_input(input_quantities, "temp")},
           theta{get_input(input_quantities, "theta")},
           upperT{get_input(input_quantities, "upperT")},
-          vmax1{get_input(input_quantities, "vmax1")},
+          Vcmax_at_25{get_input(input_quantities, "Vcmax_at_25")},
           windspeed{get_input(input_quantities, "windspeed")},
 
           // Get pointers to output quantities
-          canopy_assimilation_rate_CO2_op{get_op(output_quantities, "canopy_assimilation_rate_CO2")},
-          canopy_transpiration_rate_op{get_op(output_quantities, "canopy_transpiration_rate")},
+          canopy_assimilation_molar_flux_op{get_op(output_quantities, "canopy_assimilation_molar_flux")},
           canopy_conductance_op{get_op(output_quantities, "canopy_conductance")},
-          GrossAssim_CO2_op{get_op(output_quantities, "GrossAssim_CO2")},
-          canopy_photorespiration_rate_CO2_op{get_op(output_quantities, "canopy_photorespiration_rate_CO2")}
+          canopy_gross_assimilation_molar_flux_op{get_op(output_quantities, "canopy_gross_assimilation_molar_flux")},
+          canopy_non_photorespiratory_CO2_release_rate_op{get_op(output_quantities, "canopy_non_photorespiratory_CO2_release_molar_flux")},
+          canopy_photorespiration_molar_flux_op{get_op(output_quantities, "canopy_photorespiration_molar_flux")},
+          canopy_transpiration_rate_op{get_op(output_quantities, "canopy_transpiration_rate")},
+          whole_plant_growth_respiration_molar_flux_op{get_op(output_quantities, "whole_plant_growth_respiration_molar_flux")}
     {
     }
     static string_vector get_inputs();
@@ -91,17 +94,18 @@ class c4_canopy : public direct_module
     double const& chil;
     double const& cosine_zenith_angle;
     double const& gbw_canopy;
+    double const& growth_respiration_fraction;
     double const& Gs_min;
+    double const& k_diffuse;
     double const& kparm;
     double const& kpLN;
-    double const& k_diffuse;
     double const& lai;
-    double const& LeafN;
-    double const& leafwidth;
     double const& leaf_reflectance_nir;
     double const& leaf_reflectance_par;
     double const& leaf_transmittance_nir;
     double const& leaf_transmittance_par;
+    double const& LeafN;
+    double const& leafwidth;
     double const& lnfun;
     double const& lowerT;
     double const& nalphab0;
@@ -118,22 +122,24 @@ class c4_canopy : public direct_module
     double const& nvmaxb1;
     double const& par_energy_content;
     double const& par_energy_fraction;
-    double const& Rd;
     double const& rh;
+    double const& RL_at_25;
     double const& solar;
     double const& StomataWS;
     double const& temp;
     double const& theta;
     double const& upperT;
-    double const& vmax1;
+    double const& Vcmax_at_25;
     double const& windspeed;
 
     // Pointers to output quantities
-    double* canopy_assimilation_rate_CO2_op;
-    double* canopy_transpiration_rate_op;
+    double* canopy_assimilation_molar_flux_op;
     double* canopy_conductance_op;
-    double* GrossAssim_CO2_op;
-    double* canopy_photorespiration_rate_CO2_op;
+    double* canopy_gross_assimilation_molar_flux_op;
+    double* canopy_non_photorespiratory_CO2_release_rate_op;
+    double* canopy_photorespiration_molar_flux_op;
+    double* canopy_transpiration_rate_op;
+    double* whole_plant_growth_respiration_molar_flux_op;
 
     // Main operation
     void do_operation() const;
@@ -144,29 +150,30 @@ string_vector c4_canopy::get_inputs()
     return {
         "absorbed_longwave",  // J / m^2 / s
         "alpha1",
-        "atmospheric_pressure",       // Pa
-        "atmospheric_scattering",     // dimensionless
-        "atmospheric_transmittance",  // dimensionless
-        "b0",                         // mol / m^2 / s
-        "b1",                         // dimensionless
-        "beta",                       // dimensionless
-        "Catm",                       // ppm
-        "chil",                       // dimensionless
-        "cosine_zenith_angle",        // dimensionless
-        "gbw_canopy",                 // m / s
-        "Gs_min",                     // mol / m^2 / s
+        "atmospheric_pressure",         // Pa
+        "atmospheric_scattering",       // dimensionless
+        "atmospheric_transmittance",    // dimensionless
+        "b0",                           // mol / m^2 / s
+        "b1",                           // dimensionless
+        "beta",                         // dimensionless
+        "Catm",                         // ppm
+        "chil",                         // dimensionless
+        "cosine_zenith_angle",          // dimensionless
+        "gbw_canopy",                   // m / s
+        "growth_respiration_fraction",  // dimensionless
+        "Gs_min",                       // mol / m^2 / s
+        "k_diffuse",                    // dimensionless
         "kparm",
         "kpLN",
-        "k_diffuse",  // dimensionless
-        "lai",        // dimensionless from m^2 leaf / m^2 ground
-        "LeafN",
-        "leafwidth",               // m
+        "lai",                     // dimensionless from m^2 leaf / m^2 ground
         "leaf_reflectance_nir",    // dimensionless
         "leaf_reflectance_par",    // dimensionless
         "leaf_transmittance_nir",  // dimensionless
         "leaf_transmittance_par",  // dimensionless
-        "lnfun",                   // not a physical quantity
-        "lowerT",                  // degrees C
+        "LeafN",
+        "leafwidth",  // m
+        "lnfun",      // not a physical quantity
+        "lowerT",     // degrees C
         "nalphab0",
         "nalphab1",
         "nileafn",
@@ -181,14 +188,14 @@ string_vector c4_canopy::get_inputs()
         "nvmaxb1",
         "par_energy_content",   // J / micromol
         "par_energy_fraction",  // dimensionless
-        "Rd",                   // micromol / m^2 / s
         "rh",                   // dimensionless from Pa / Pa
+        "RL_at_25",             // micromol / m^2 / s
         "solar",                // micromol / m^2 / s
         "StomataWS",            // dimensionless
         "temp",                 // degrees C
         "theta",                // dimensionless
         "upperT",               // degrees C
-        "vmax1",                // micromol / m^2 / s
+        "Vcmax_at_25",          // micromol / m^2 / s
         "windspeed"             // m / s
     };
 }
@@ -196,11 +203,13 @@ string_vector c4_canopy::get_inputs()
 string_vector c4_canopy::get_outputs()
 {
     return {
-        "canopy_assimilation_rate_CO2",     // micromol CO2 / m^2 / s
-        "canopy_transpiration_rate",        // Mg / ha / hr
-        "canopy_conductance",               // mol / m^2 / s
-        "GrossAssim_CO2",                   // micromol CO2 / m^2 / s
-        "canopy_photorespiration_rate_CO2"  // micromol / m^2 / s
+        "canopy_assimilation_molar_flux",                      // micromol / m^2 / s
+        "canopy_conductance",                                  // mol / m^2 / s
+        "canopy_gross_assimilation_molar_flux",                // micromol / m^2 / s
+        "canopy_non_photorespiratory_CO2_release_molar_flux",  // micromol / m^2 / s
+        "canopy_photorespiration_molar_flux",                  // micromol / m^2 / s
+        "canopy_transpiration_rate",                           // Mg / ha / hr
+        "whole_plant_growth_respiration_molar_flux"            // micromol / m^2 / s
     };
 }
 
@@ -235,37 +244,40 @@ void c4_canopy::do_operation() const
         chil,
         cosine_zenith_angle,
         gbw_canopy,
+        growth_respiration_fraction,
         Gs_min,
+        k_diffuse,
         kparm,
         kpLN,
-        k_diffuse,
         lai,
-        LeafN,
-        leafwidth,
         leaf_reflectance_nir,
         leaf_reflectance_par,
         leaf_transmittance_nir,
         leaf_transmittance_par,
+        LeafN,
+        leafwidth,
         lowerT,
         par_energy_content,
         par_energy_fraction,
-        Rd,
         rh,
+        RL_at_25,
         solar,
         StomataWS,
         theta,
         upperT,
-        vmax1,
+        Vcmax_at_25,
         windspeed,
         lnfun,
         nlayers);
 
     // Update the parameter list
-    update(canopy_assimilation_rate_CO2_op, can_result.Assim);     // micromol / m^2 /s
-    update(canopy_transpiration_rate_op, can_result.Trans);        // Mg / ha / hr
-    update(canopy_conductance_op, can_result.canopy_conductance);  // mol / m^2 / s
-    update(GrossAssim_CO2_op, can_result.GrossAssim);              // micromol / m^2 /s
-    update(canopy_photorespiration_rate_CO2_op, can_result.Rp);    // micromol / m^2 /s
+    update(canopy_assimilation_molar_flux_op, can_result.Assim);                      // micromol / m^2 /s
+    update(canopy_conductance_op, can_result.canopy_conductance);                     // mol / m^2 / s
+    update(canopy_gross_assimilation_molar_flux_op, can_result.GrossAssim);           // micromol / m^2 /s
+    update(canopy_non_photorespiratory_CO2_release_rate_op, can_result.RL);           // micromol / m^2 / s
+    update(canopy_photorespiration_molar_flux_op, can_result.Rp);                     // micromol / m^2 /s
+    update(canopy_transpiration_rate_op, can_result.Trans);                           // Mg / ha / hr
+    update(whole_plant_growth_respiration_molar_flux_op, can_result.whole_plant_gr);  // micromol / m^2 / s
 }
 }  // namespace standardBML
 #endif
