@@ -5,6 +5,7 @@
 #include "../framework/state_map.h"
 #include "../framework/module.h"
 #include "../framework/constants.h"  // for molar_mass_of_water, molar_mass_of_glucose
+#include "respiration.h"             // for growth_resp
 
 namespace standardBML
 {
@@ -33,29 +34,33 @@ class multilayer_canopy_integrator : public direct_module
           nlayers(nlayers),
 
           // Get pointers to input quantities
-          sunlit_fraction_ips{get_multilayer_ip(input_quantities, nlayers, "sunlit_fraction")},
-          sunlit_Assim_ips{get_multilayer_ip(input_quantities, nlayers, "sunlit_Assim")},
-          sunlit_GrossAssim_ips{get_multilayer_ip(input_quantities, nlayers, "sunlit_GrossAssim")},
-          sunlit_Gs_ips{get_multilayer_ip(input_quantities, nlayers, "sunlit_Gs")},
-          sunlit_Rp_ips{get_multilayer_ip(input_quantities, nlayers, "sunlit_Rp")},
-          sunlit_TransR_ips{get_multilayer_ip(input_quantities, nlayers, "sunlit_TransR")},
-          shaded_fraction_ips{get_multilayer_ip(input_quantities, nlayers, "shaded_fraction")},
           shaded_Assim_ips{get_multilayer_ip(input_quantities, nlayers, "shaded_Assim")},
+          shaded_fraction_ips{get_multilayer_ip(input_quantities, nlayers, "shaded_fraction")},
           shaded_GrossAssim_ips{get_multilayer_ip(input_quantities, nlayers, "shaded_GrossAssim")},
           shaded_Gs_ips{get_multilayer_ip(input_quantities, nlayers, "shaded_Gs")},
+          shaded_RL_ips{get_multilayer_ip(input_quantities, nlayers, "shaded_RL")},
           shaded_Rp_ips{get_multilayer_ip(input_quantities, nlayers, "shaded_Rp")},
           shaded_TransR_ips{get_multilayer_ip(input_quantities, nlayers, "shaded_TransR")},
+          sunlit_Assim_ips{get_multilayer_ip(input_quantities, nlayers, "sunlit_Assim")},
+          sunlit_fraction_ips{get_multilayer_ip(input_quantities, nlayers, "sunlit_fraction")},
+          sunlit_GrossAssim_ips{get_multilayer_ip(input_quantities, nlayers, "sunlit_GrossAssim")},
+          sunlit_Gs_ips{get_multilayer_ip(input_quantities, nlayers, "sunlit_Gs")},
+          sunlit_RL_ips{get_multilayer_ip(input_quantities, nlayers, "sunlit_RL")},
+          sunlit_Rp_ips{get_multilayer_ip(input_quantities, nlayers, "sunlit_Rp")},
+          sunlit_TransR_ips{get_multilayer_ip(input_quantities, nlayers, "sunlit_TransR")},
 
           // Get references to input quantities
-          lai{get_input(input_quantities, "lai")},
           growth_respiration_fraction{get_input(input_quantities, "growth_respiration_fraction")},
+          lai{get_input(input_quantities, "lai")},
 
           // Get pointers to output quantities
-          canopy_assimilation_rate_CO2_op{get_op(output_quantities, "canopy_assimilation_rate_CO2")},
-          canopy_transpiration_rate_op{get_op(output_quantities, "canopy_transpiration_rate")},
+          canopy_assimilation_molar_flux_op{get_op(output_quantities, "canopy_assimilation_molar_flux")},
           canopy_conductance_op{get_op(output_quantities, "canopy_conductance")},
-          GrossAssim_CO2_op{get_op(output_quantities, "GrossAssim_CO2")},
-          canopy_photorespiration_rate_CO2_op{get_op(output_quantities, "canopy_photorespiration_rate_CO2")}
+          canopy_gross_assimilation_molar_flux_op{get_op(output_quantities, "canopy_gross_assimilation_molar_flux")},
+          canopy_photorespiration_molar_flux_op{get_op(output_quantities, "canopy_photorespiration_molar_flux")},
+          canopy_RL_molar_flux_op{get_op(output_quantities, "canopy_non_photorespiratory_CO2_release_molar_flux")},
+          canopy_transpiration_rate_op{get_op(output_quantities, "canopy_transpiration_rate")},
+          whole_plant_growth_respiration_molar_flux_op{get_op(output_quantities, "whole_plant_growth_respiration_molar_flux")}
     {
     }
 
@@ -64,29 +69,33 @@ class multilayer_canopy_integrator : public direct_module
     int const nlayers;
 
     // Pointers to input quantities
-    std::vector<double const*> const sunlit_fraction_ips;
-    std::vector<double const*> const sunlit_Assim_ips;
-    std::vector<double const*> const sunlit_GrossAssim_ips;
-    std::vector<double const*> const sunlit_Gs_ips;
-    std::vector<double const*> const sunlit_Rp_ips;
-    std::vector<double const*> const sunlit_TransR_ips;
-    std::vector<double const*> const shaded_fraction_ips;
     std::vector<double const*> const shaded_Assim_ips;
+    std::vector<double const*> const shaded_fraction_ips;
     std::vector<double const*> const shaded_GrossAssim_ips;
     std::vector<double const*> const shaded_Gs_ips;
+    std::vector<double const*> const shaded_RL_ips;
     std::vector<double const*> const shaded_Rp_ips;
     std::vector<double const*> const shaded_TransR_ips;
+    std::vector<double const*> const sunlit_Assim_ips;
+    std::vector<double const*> const sunlit_fraction_ips;
+    std::vector<double const*> const sunlit_GrossAssim_ips;
+    std::vector<double const*> const sunlit_Gs_ips;
+    std::vector<double const*> const sunlit_RL_ips;
+    std::vector<double const*> const sunlit_Rp_ips;
+    std::vector<double const*> const sunlit_TransR_ips;
 
     // References to input quantities
-    double const& lai;
     double const& growth_respiration_fraction;
+    double const& lai;
 
     // Pointers to output quantities
-    double* canopy_assimilation_rate_CO2_op;
-    double* canopy_transpiration_rate_op;
+    double* canopy_assimilation_molar_flux_op;
     double* canopy_conductance_op;
-    double* GrossAssim_CO2_op;
-    double* canopy_photorespiration_rate_CO2_op;
+    double* canopy_gross_assimilation_molar_flux_op;
+    double* canopy_photorespiration_molar_flux_op;
+    double* canopy_RL_molar_flux_op;
+    double* canopy_transpiration_rate_op;
+    double* whole_plant_growth_respiration_molar_flux_op;
 
     // Main operation
     virtual void do_operation() const;
@@ -107,26 +116,28 @@ string_vector multilayer_canopy_integrator::get_inputs(int nlayers)
 {
     // Define the multilayer inputs
     string_vector multilayer_inputs = {
-        "sunlit_fraction",    // dimensionless
-        "sunlit_Assim",       // micromol / m^2 /s
-        "sunlit_GrossAssim",  // micromol / m^2 /s
-        "sunlit_Gs",          // mmol / m^2 / s
-        "sunlit_Rp",          // micromol / m^2 /s
-        "sunlit_TransR",      // mmol / m^2 / s
-        "shaded_fraction",    // dimensionless
         "shaded_Assim",       // micromol / m^2 /s
+        "shaded_fraction",    // dimensionless
         "shaded_GrossAssim",  // micromol / m^2 /s
         "shaded_Gs",          // mmol / m^2 / s
+        "shaded_RL",          // micromol / m^2 /s
         "shaded_Rp",          // micromol / m^2 /s
         "shaded_TransR",      // mmol / m^2 / s
+        "sunlit_Assim",       // micromol / m^2 /s
+        "sunlit_fraction",    // dimensionless
+        "sunlit_GrossAssim",  // micromol / m^2 /s
+        "sunlit_Gs",          // mmol / m^2 / s
+        "sunlit_RL",          // micromol / m^2 /s
+        "sunlit_Rp",          // micromol / m^2 /s
+        "sunlit_TransR"       // mmol / m^2 / s
     };
 
     // Get the full list by appending layer numbers
     string_vector all_inputs = generate_multilayer_quantity_names(nlayers, multilayer_inputs);
 
     // Add any other inputs
-    all_inputs.push_back("lai");                          // dimensionless from m^2 / m^2
     all_inputs.push_back("growth_respiration_fraction");  // dimensionless
+    all_inputs.push_back("lai");                          // dimensionless from m^2 / m^2
 
     return all_inputs;
 }
@@ -137,11 +148,13 @@ string_vector multilayer_canopy_integrator::get_inputs(int nlayers)
 string_vector multilayer_canopy_integrator::get_outputs(int /*nlayers*/)
 {
     return {
-        "canopy_assimilation_rate_CO2",     // micromol CO2 / m^2 / s
-        "canopy_transpiration_rate",        // Mg / ha / hr
-        "canopy_conductance",               // mmol / m^2 / s
-        "GrossAssim_CO2",                   // micromol CO2 / m^2 / s
-        "canopy_photorespiration_rate_CO2"  // micromol CO2 / m^2 / s
+        "canopy_assimilation_molar_flux",                      // micromol CO2 / m^2 / s
+        "canopy_conductance",                                  // mmol / m^2 / s
+        "canopy_gross_assimilation_molar_flux",                // micromol CO2 / m^2 / s
+        "canopy_non_photorespiratory_CO2_release_molar_flux",  // micromol CO2 / m^2 / s
+        "canopy_photorespiration_molar_flux",                  // micromol CO2 / m^2 / s
+        "canopy_transpiration_rate",                           // Mg / ha / hr
+        "whole_plant_growth_respiration_molar_flux"            // micromol CO2 / m^2 / s
     };
 }
 
@@ -153,11 +166,13 @@ void multilayer_canopy_integrator::do_operation() const
 void multilayer_canopy_integrator::run() const
 {
     double const LAIc = lai / nlayers;
+
     double canopy_assimilation_rate{0.0};
-    double canopy_transpiration_rate{0.0};
     double canopy_conductance{0.0};
-    double GrossAssim{0.0};
+    double canopy_gross_assimilation_rate{0.0};
     double canopy_photorespiration_rate{0.0};
+    double canopy_RL{0.0};
+    double canopy_transpiration_rate{0.0};
 
     // Integrate assimilation, transpiration, and conductance throughout the
     // canopy
@@ -174,22 +189,19 @@ void multilayer_canopy_integrator::run() const
         canopy_conductance += *sunlit_Gs_ips[i] * sunlit_lai +
                               *shaded_Gs_ips[i] * shaded_lai;
 
-        GrossAssim += *sunlit_GrossAssim_ips[i] * sunlit_lai +
-                      *shaded_GrossAssim_ips[i] * shaded_lai;
+        canopy_gross_assimilation_rate += *sunlit_GrossAssim_ips[i] * sunlit_lai +
+                                          *shaded_GrossAssim_ips[i] * shaded_lai;
 
         canopy_photorespiration_rate += *sunlit_Rp_ips[i] * sunlit_lai +
                                         *shaded_Rp_ips[i] * shaded_lai;
+
+        canopy_RL += *sunlit_RL_ips[i] * sunlit_lai +
+                     *shaded_RL_ips[i] * shaded_lai;
     }
 
-    // Modify net assimilation to account for respiration
-    // Note: this was originally only done for the C3 canopy
-    // Note: it seems like this should not be necessary since the assimilation
-    //   model includes respiration
-    double const growth_respiration =
-        growth_respiration_fraction * fabs(canopy_assimilation_rate);  // micromol / m^2 / s
-
-    canopy_assimilation_rate =
-        canopy_assimilation_rate - growth_respiration;  // micromol / m^2 / s
+    // Calculate the rate of whole-plant growth respiration
+    double const whole_plant_gr =
+        growth_resp(canopy_assimilation_rate, growth_respiration_fraction);  // micromol / m^2 / s
 
     // For transpiration, we need to convert mmol / m^2 / s into Mg / ha / hr
     // using the molar mass of water in kg / mol, which can be accomplished by
@@ -198,15 +210,13 @@ void multilayer_canopy_integrator::run() const
     // = 36 s * mol * Mg * m^2 / (hr * mmol * kg * ha)
     const double cf2 = physical_constants::molar_mass_of_water * 36;  // (Mg / ha / hr) / (mmol / m^2 / s)
 
-    update(canopy_assimilation_rate_CO2_op, canopy_assimilation_rate);
-
-    update(GrossAssim_CO2_op, GrossAssim);
-
-    update(canopy_transpiration_rate_op, canopy_transpiration_rate * cf2);
-
+    update(canopy_assimilation_molar_flux_op, canopy_assimilation_rate - whole_plant_gr);
     update(canopy_conductance_op, canopy_conductance);
-
-    update(canopy_photorespiration_rate_CO2_op, canopy_photorespiration_rate);
+    update(canopy_gross_assimilation_molar_flux_op, canopy_gross_assimilation_rate);
+    update(canopy_photorespiration_molar_flux_op, canopy_photorespiration_rate);
+    update(canopy_RL_molar_flux_op, canopy_RL);
+    update(canopy_transpiration_rate_op, canopy_transpiration_rate * cf2);
+    update(whole_plant_growth_respiration_molar_flux_op, whole_plant_gr);
 }
 
 ////////////////////////////////////////
