@@ -17,14 +17,11 @@ class parameter_calculator : public direct_module
           Leaf{get_input(input_quantities, "Leaf")},
           LeafN_0{get_input(input_quantities, "LeafN_0")},
           LeafN{get_input(input_quantities, "LeafN")},
-          vmax_n_intercept{get_input(input_quantities, "vmax_n_intercept")},
-          vmax1{get_input(input_quantities, "vmax1")},
           alphab1{get_input(input_quantities, "alphab1")},
           alpha1{get_input(input_quantities, "alpha1")},
 
           // Get pointers to output quantities
           lai_op{get_op(output_quantities, "lai")},
-          vmax_op{get_op(output_quantities, "vmax")},
           alpha_op{get_op(output_quantities, "alpha")}
     {
     }
@@ -38,14 +35,11 @@ class parameter_calculator : public direct_module
     double const& Leaf;
     double const& LeafN_0;
     double const& LeafN;
-    double const& vmax_n_intercept;
-    double const& vmax1;
     double const& alphab1;
     double const& alpha1;
 
     // Pointers to output quantities
     double* lai_op;
-    double* vmax_op;
     double* alpha_op;
 
     // Main operation
@@ -55,14 +49,12 @@ class parameter_calculator : public direct_module
 string_vector parameter_calculator::get_inputs()
 {
     return {
-        "Sp",                // Ha / Mg
-        "Leaf",              // Mg / Ha
-        "LeafN_0",           //
-        "LeafN",             //
-        "vmax_n_intercept",  //
-        "vmax1",             // micromol / m^2 / s
-        "alphab1",           //
-        "alpha1"             //
+        "Sp",       // Ha / Mg
+        "Leaf",     // Mg / Ha
+        "LeafN_0",  //
+        "LeafN",    //
+        "alphab1",  //
+        "alpha1"    //
     };
 }
 
@@ -70,7 +62,6 @@ string_vector parameter_calculator::get_outputs()
 {
     return {
         "lai",   // dimensionless
-        "vmax",  // micromol / m^2 / s
         "alpha"  //
     };
 }
@@ -78,7 +69,6 @@ string_vector parameter_calculator::get_outputs()
 void parameter_calculator::do_operation() const
 {
     update(lai_op, Leaf * Sp);
-    update(vmax_op, (LeafN_0 - LeafN) * vmax_n_intercept + vmax1);
     update(alpha_op, (LeafN_0 - LeafN) * alphab1 + alpha1);
 }
 
